@@ -1,9 +1,11 @@
+#!/usr/local/bin/perl -w
 # -*- perl -*-
 
 #
+# $Id: WidgetDump.pm,v 1.29 2004/01/14 22:52:15 eserte Exp $
 # Author: Slaven Rezic
 #
-# Copyright (C) 1999-2011,2013 Slaven Rezic. All rights reserved.
+# Copyright (C) 1999-2004 Slaven Rezic. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -15,7 +17,7 @@ package Tk::WidgetDump;
 use vars qw($VERSION);
 use strict;
 
-$VERSION = '1.38_51';
+$VERSION = sprintf("%d.%02d", q$Revision: 1.29 $ =~ /(\d+)\.(\d+)/);
 
 package # hide from CPAN indexer
   Tk::Widget;
@@ -83,7 +85,6 @@ sub WidgetDump {
 			    }, Ev('X'), Ev('Y') ]);
     }
 
-    $t;
 }
 
 sub WD_HList {
@@ -291,16 +292,10 @@ sub WidgetInfo {
     $txt->insert("end", "Configuration:\n\n", "title");
     $txt->insert("end", "Option Switch\tOptionDB Name\tOptionDB Class\tDefault Value\tCurrent Value\n", "title");
     foreach my $c ($w->configure) {
-	my $class = $c->[2];
-	my $name  = $c->[1];
-	if ($name =~ m{^-}) { # an alias
-	    my @c_alias = $w->configure($name);
-	    $class = $c_alias[2];
-	}
 	$txt->insert("end",
 		     join("\t", map { !defined $_ ? "<undef>" : $_ } @$c),
 		     ["widgetlink",
-		      "config-" . $w . ($c->[0]||"") . "-" . ($class||"")],
+		      "config-" . $w . ($c->[0]||"") . "-" . ($c->[2]||"")],
 		     "\n");
     }
     $txt->insert("end", "\n");
@@ -319,7 +314,6 @@ sub WidgetInfo {
 
     $Tk::WidgetDump::ref2widget{$w} = $w;
 
-    $txt->insert("end", "Self:\t" . $w . "\n");
     if (defined $w->parent) {
 	$txt->insert("end", "Parent:\t" . $w->parent,
 		     ["widgetlink", "href-" . $w->parent], "\n");
@@ -458,28 +452,25 @@ sub WidgetInfo {
 			 -command => sub {
 			     $wd->method_call($w);
 			 });
-    $txt->windowCreate("end", -window => $b);
+    $txt->windowCreate("end",
+		       -window => $b,
+		       );
 
     if ($w->isa('Tk::Canvas')) {
 	my $b = $txt->Button(-text => "Canvas dump",
 			     -command => sub {
 				 $wd->canvas_dump($w);
 			     });
-	$txt->windowCreate("end", -window => $b);
-    }
-    if ($w->can('tagNames')) {
-	my $b = $txt->Button(-text => 'Tags',
-			     -command => sub {
-				 $wd->tag_dump($w);
-			     });
-	$txt->windowCreate('end', -window => $b);
+	$txt->windowCreate("end",
+			   -window => $b,
+			  );
     }
 
     my $ObjScanner;
     if (!eval {
 		require Tk::ObjEditor;
 		$ObjScanner = "ObjEditor";
-		$Storable::forgive_me = $Storable::forgive_me = 1; # XXX hack to prevent problems with code refs
+		$Storable::forgive_me = 1; # XXX hack to prevent problems with code refs
 		1;
 	    }) {
 	eval { require Tk::ObjScanner;
@@ -517,8 +508,8 @@ sub WidgetInfo {
 sub show_bindings {
     my($wd, $w) = @_;
     my $t = $wd->Toplevel(-title => 'Bindings');
-    my $ttxt = $t->Scrolled($wd->_more_widget_class)->pack(-fill => 'both',
-							   -expand => 1);
+    my $ttxt = $t->Scrolled('ROText')->pack(-fill => 'both',
+					    -expand => 1);
     _text_link_config($ttxt, sub { _bind_text_tag($_[0], $wd) } );
     foreach my $bindtag ($w->bindtags) {
 	$ttxt->insert("end", "Bind tag: $bindtag\n\n");
@@ -543,17 +534,11 @@ sub show_bindings {
 sub show_binding_details {
     my($wd, $widget, $bindtag, $bind) = @_;
     my $t = $wd->Toplevel(-title => "Binding details");
-    my $ttxt = $t->Scrolled($wd->_more_widget_class)->pack(-fill => "both", -expand => 1);
+    my $ttxt = $t->Scrolled("ROText")->pack(-fill => "both", -expand => 1);
     my $cb = $widget->Tk::bind($bindtag, $bind);
     $ttxt->insert("end", "Binding <$bind> for bindtag <$bindtag>:\n");
     require Data::Dumper;
-    my $txt;
-    my $dd = Data::Dumper->new([$cb],[]);
-    if ($dd->can("Deparse")) {
-	$txt = $dd->Deparse(1)->Useqq(1)->Dump;
-    } else {
-	$txt = "Sorry, your version of Data::Dumper is not capable to deparse the CODE reference.";
-    }
+    my $txt = Data::Dumper->new([$cb],[])->Deparse(1)->Useqq(1)->Dump;
     $ttxt->insert("end", $txt);
 }
 
@@ -605,17 +590,6 @@ sub _edit_config {
 	$e = eval 'Tk::WidgetDump::Entry->entry($t, \$val, $set_sub)';
 	warn $@ if $@;
     }
-#XXX ja?
-#     $t->Button(-text => "Undef and close",
-# 	       -command => sub {
-# 		   $val = undef;
-# 		   $set_sub->();
-# 		   $t->destroy;
-# 	       }
-# 	      )->pack(-side => "left");
-    $t->Button(-text => "Set",
-	       -command => $set_sub,
-	      )->pack(-side => "left");
     $t->Button(-text => "Close",
 	       -command => [$t, 'destroy'],
 	      )->pack(-side => "left");
@@ -629,7 +603,7 @@ sub method_call {
     my $t = $wd->Toplevel(-title => "Method call");
     my $f = $t->Frame->pack(-fill => "x");
     my $eval;
-    $f->Label(-text => "Method call on $w ->")->pack(-side => "left");
+    $f->Label(-text => "Method call on $w")->pack(-side => "left");
     my $e = $f->_hist_entry({-textvariable => \$eval},
 			    {-match => 1, -dup => 0})->pack(-side => "left");
     $e->focus;
@@ -649,15 +623,10 @@ sub method_call {
 	$text->delete("1.0", "end");
 	$text->insert("end", $res);
     };
-    my $close = sub {
-	$t->destroy;
-    };
-    $e->bind('<Return>' => $doit);
-    $e->bind('<Escape>' => $close);
+    $e->bind("<Return>" => $doit);
     $f->Button(-text => "Execute!", -command => $doit)->pack(-side => "left");
-    $f->Button(-text => "Close", -command => $close)->pack(-side => "left");
-    $text = $t->Scrolled($wd->_more_widget_class,
-			 -scrollbars => "osoe",
+    $f->Button(-text => "Close", -command => [$t, "destroy"])->pack(-side => "left");
+    $text = $t->Scrolled("ROText", -scrollbars => "osoe",
 			 -font => "courier 10", # XXX do not hardcode
 			 -width => 40, -height => 5)->pack(-fill => "both", -expand => 1);
 }
@@ -695,13 +664,11 @@ sub _text_link_config {
     $txt->tagConfigure("title", -font => "Helvetica 10 bold"); # XXX do not hardcode!
 }
 
-######################################################################
-# Canvas
 sub canvas_config {
     my($wd, $c, $item) = @_;
     my $t = $wd->Toplevel(-title => "Canvas config of item $item");
 
-    my $txt = $t->Scrolled($wd->_more_widget_class,
+    my $txt = $t->Scrolled("ROText",
 			   -tabs => [map { (5*$_) . "c" } (1 .. 8)],
 			   -scrollbars => "osow",
 			   -wrap => "none",
@@ -729,8 +696,8 @@ sub canvas_config {
 sub canvas_dump {
     my($wd, $c) = @_;
     my $t = $wd->Toplevel(-title => "Canvas dump of $c");
-    my $txt = $t->Scrolled($wd->_more_widget_class,
-			   -scrollbars => "osow",
+    require Tk::ROText;
+    my $txt = $t->Scrolled("ROText", -scrollbars => "osow",
 			   -tabs => [map { (3*$_) . "c" } (1 .. 3)],
 			  )->pack(-fill => "both", -expand => 1);
     _text_link_config($txt, sub { _bind_text_tag($_[0], $wd) } );
@@ -772,14 +739,6 @@ sub edit_canvas_config {
     warn $@ if $@;
     $e->focus if Tk::Exists($e);
     $t->bind("<Escape>" => [$t, 'destroy']);
-#XXX ja?
-#     $t->Button(-text => "Undef and close",
-# 	       -command => sub {
-# 		   $val = undef;
-# 		   $set_sub->();
-# 		   $t->destroy;
-# 	       }
-# 	      )->pack(-side => "left");
     $t->Button(-text => "Close", -command => [$t, "destroy"])->pack(-side => "left");
 }
 
@@ -794,6 +753,7 @@ sub edit_canvas_coords {
 	warn $@;
 	return;
     }
+    my $oldval = $val;
 
     my $t = $wd->Toplevel(-title => "Edit canvas coords");
     my $set_sub = sub {
@@ -812,75 +772,6 @@ sub edit_canvas_coords {
     $t->Button(-text => "Close", -command => [$t, "destroy"]);
 }
 
-######################################################################
-# Tags
-sub tag_dump {
-    my($wd, $w) = @_;
-    my $t = $wd->Toplevel(-title => "Tag dump of $w");
-    my $txt = $t->Scrolled($wd->_more_widget_class,
-			   -width => 15,
-			   -height => 5,
-			   -scrollbars => 'osow',
-			  )->pack(-fill => "both", -expand => 1);
-    _text_link_config($txt, sub { _bind_text_tag($_[0], $wd) } );
-
-    $txt->insert('end', "Tags\n\n", 'title');
-    for my $tag_name ($w->tagNames) {
-	$txt->insert('end', $tag_name, ["widgetlink", "tag-$w-$tag_name"], "\n");
-    }
-}
-
-sub tag_options {
-    my($wd, $w, $tag) = @_;
-
-    my $t = $wd->Toplevel(-title => "Options for tag $tag");
-    my $txt = $t->Scrolled($wd->_more_widget_class,
-			   -scrollbars => 'osow',
-			   -tabs => [map { (3*$_) . "c" } (1 .. 3)],
-			  )->pack(-fill => "both", -expand => 1);
-    _text_link_config($txt, sub { _bind_text_tag($_[0], $wd) } );
-    $txt->insert('end', "Option Name\tDefault Value\tCurrent Value\n", 'title');
-    for my $option_def ($w->tagConfigure($tag)) {
-	my($key,undef,undef,$def_val,$curr_val) = @$option_def;
-	$txt->insert('end',
-		     join("\t", map { !defined $_ ? '<undef>' : $_ } ($key, $def_val, $curr_val)),
-		     ['widgetlink', "tag-config-$w-$tag-$key"],
-		     "\n");
-    }
-    $txt->insert('end', "\n");
-}
-
-sub edit_tag_option {
-    my($wd, $w, $tag, $key) = @_;
-
-    my $val;
-    eval {
-	$val = $w->tagCget($tag, $key);
-    };
-    if ($@) {
-	warn $@;
-	return;
-    }
-
-    my $t = $wd->Toplevel(-title => "Edit tag option $key");
-    my $set_sub = sub {
-	eval {
-	    $w->tagConfigure($tag, $key, $val);
-	};
-	warn $@ if $@;
-    };
-
-    $t->Label(-text => "Edit option $key for tag $tag:")->pack(-side => "left");
-    my $e;
-    $e = eval 'Tk::WidgetDump::Entry->entry($t, \$val, $set_sub)';
-    warn $@ if $@;
-    $e->focus if Tk::Exists($e);
-    $t->bind("<Escape>" => [$t, 'destroy']);
-    $t->Button(-text => "Close", -command => [$t, "destroy"]);
-}
-
-######################################################################
-# Misc
 sub _insert_wd {
     my($wd, $hl, $top, $par) = @_;
     my $i = 0;
@@ -1017,28 +908,6 @@ sub _bind_text_tag {
 	    return;
 	}
     }
-
-    $i = _lsearch('tag\-config\-.*', @tags);
-    if ($i >= 0) {
-	if ($tags[$i] =~ /^tag-config-([^-]+)-(.*)-(-.+)$/) {
-	    my($w_name, $tag_name, $key) = ($1, $2, $3);
-	    my $widget = $ref2widget{$w_name};
-	    $wd->edit_tag_option($widget, $tag_name, $key);
-	    return;
-	}
-    }
-
-    $i = _lsearch('tag\-.*', @tags);
-    if ($i >= 0) {
-	if ($tags[$i] =~ /^tag-(.*)-(.*)$/) {
-	    my $w_name = $1;
-	    my $tag_name = $2;
-	    my $widget = $ref2widget{$w_name};
-	    $wd->tag_options($widget, $tag_name);
-	    return;
-	}
-    }
-
     warn "Can't match $tags[$i]";
 }
 
@@ -1059,9 +928,10 @@ sub _get_widget_info_window {
 	$wi->geometry("930x450");
     }
 
+    require Tk::ROText;
     my $bf = $wi->Frame->pack(-fill => 'x', -side => "bottom");
 
-    my $txt = $wi->Scrolled($wd->_more_widget_class,
+    my $txt = $wi->Scrolled("ROText",
 			    -tabs => [map { (5*$_) . "c" } (1 .. 8)],
 			    -wrap => "none",
 			   )->pack(-expand => 1, -fill => "both");
@@ -1096,19 +966,6 @@ sub _lsearch {
 
 } # end lsearch
 
-{
-    my $more_widget_class;
-    sub _more_widget_class {
-	return $more_widget_class if $more_widget_class;
-	if (eval { require Tk::More; 1 }) {
-	    return $more_widget_class = 'More';
-	} else {
-	    require Tk::ROText;
-	    return $more_widget_class = 'ROText';
-	}
-    }
-}
-
 # XXX weitermachen
 # die Idee: die gesamten Konfigurationsdaten aller Widgets per configure
 # feststellen und als String schreiben. Und das fr alle Children des
@@ -1148,15 +1005,7 @@ package # hide from CPAN indexer
   Tk::Toplevel;
 sub _WD_Characteristics {
     my $w = shift;
-    my $characteristics = eval {
-	Tk::WidgetDump::_crop($w->title) . " (" . $w->geometry . ")";
-    };
-    if ($@) {
-	# A "toplevel" which is not a real toplevel: this is true
-	# for Tk::DragDrop, see the comments there.
-	$characteristics = Tk::WidgetDump::_crop("toplevel-ish $w");
-    }
-    $characteristics;
+    Tk::WidgetDump::_crop($w->title) . " (" . $w->geometry . ")";
 }
 
 package # hide from CPAN indexer
@@ -1402,8 +1251,6 @@ sub entry {
     $f;
 }
 
-$Tk::Config::xinc = $Tk::Config::xinc if 0; # peacify -w
-
 package Tk::WidgetDump::Command;
 use base qw(Tk::WidgetDump::Entry);
 
@@ -1476,7 +1323,20 @@ use base qw(Tk::WidgetDump::Pixels);
 package Tk::WidgetDump::Underline;
 use base qw(Tk::WidgetDump::NumEntry);
 
-1;
+
+return 1 if caller;
+
+######################################################################
+
+package main;
+
+# self-test
+my $top = MainWindow->new;
+$top->Canvas->pack->createLine(0,0,100,100);
+#$top->withdraw;
+$top->WidgetDump;
+$top->WidgetDump;
+Tk::MainLoop;
 
 __END__
 
@@ -1486,15 +1346,9 @@ Tk::WidgetDump - dump the widget hierarchie
 
 =head1 SYNOPSIS
 
-In a script:
-
-    use Tk::WidgetDump; # optional
+    use Tk::WidgetDump;
     $mw = new MainWindow;
-    $mw->WidgetDump; # usually before MainLoop
-
-From the command line for a quick widget option test:
-
-    perl -MTk -MTk::WidgetDump -e '$mw=tkinit; $mw->Button->pack; $mw->WidgetDump; MainLoop'
+    $mw->WidgetDump;
 
 =head1 DESCRIPTION
 
@@ -1542,8 +1396,9 @@ itself.
 
 =item * Changing configuration values
 
-Changes are not reflected in the configuration window, you have to hit
-the "Refresh" button.
+You have to hit <Return> to see the changes. The changes are not
+reflected in the configuration window, you have to hit the "Refresh"
+button.
 
 =item * Tk::WidgetDump does not follow the conventions of a "real"
 widget (ConfiSpecs etc.)
@@ -1554,10 +1409,10 @@ widget (ConfiSpecs etc.)
 
 =head1 AUTHOR
 
-Slaven Rezic (srezic@cpan.org)
+Slaven Rezic (slaven@rezic.de)
 
 =head1 SEE ALSO
 
-L<Tk>.
+Tk(3).
 
 =cut
